<?php
/*
 * CalendarExport.php - iCal calendar export plugin for Stud.IP
 * Copyright (c) 2011  Elmar Ludwig
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'lib/calendar/lib/sync/CalendarExport.class.php';
require_once 'lib/calendar/lib/sync/CalendarWriterICalendar.class.php';

class iCal extends StudipPlugin implements SystemPlugin
{
    /**
     * plugin template factory
     */
    protected $template_factory;

    /**
     * Initialize a new instance of the plugin.
     */
    public function __construct()
    {
        global $user;

        parent::__construct();

        if (!is_object($user) || $user->id == 'nobody') {
            return;
        }

        $template_path = $this->getPluginPath() . '/templates';
        $this->template_factory = new Flexi_TemplateFactory($template_path);

        $page = basename($_SERVER['PHP_SELF']);
        $cmd = Request::option('cmd');
        $expmode = Request::option('expmode');
        $ical_enable = Request::int('ical_enable');

        if ($page === 'calendar.php' && $cmd === 'export' && $expmode === NULL) {
            $template = $this->template_factory->open('load_infobox');
            PageLayout::addHeadElement('script', array(), $template->render());

            if ($ical_enable === 1) {
                $this->set_user_key($user->id);
            } else if ($ical_enable === 0) {
                $this->clear_user_key($user->id);
            }
        }
    }

    /**
     * Return the user specific access key.
     */
    private function get_user_key($user_id)
    {
        return UserConfig::get($user_id)->getValue('ICAL_EXPORT_KEY');
    }

    /**
     * Calculate user specific access key.
     */
    private function set_user_key($user_id)
    {
        $key = '';

        for ($i = 0; $i < 32; ++$i) {
            $key .= chr(mt_rand(0, 63) + 48);
        }

        UserConfig::get($user_id)->store('ICAL_EXPORT_KEY', sha1($key));
    }

    /**
     * Clear the user specific access key.
     */
    private function clear_user_key($user_id)
    {
        UserConfig::get($user_id)->delete('ICAL_EXPORT_KEY');
    }

    private function check_user_access($username, $key)
    {
        global $template_factory;

        if (isset($username) && isset($key)) {
            $username = preg_replace('/[^\w@.-]/', '', $username);
            $user_id = get_userid($username);
            $user_key = $this->get_user_key($user_id);

            if ($user_key === NULL || $user_key !== $key) {
                throw new AccessDeniedException('invalid access key');
            }
        } else {
            $username = $_SERVER['PHP_AUTH_USER'];
            $password = $_SERVER['PHP_AUTH_PW'];

            if (isset($username) && isset($password)) {
                $result = StudipAuthAbstract::CheckAuthentication($username, $password);
            }

            if (isset($result) && $result['uid'] !== false) {
                $user_id = get_userid($username);
            } else {
                header('WWW-Authenticate: Basic realm="Stud.IP Login"');
                header('HTTP/1.1 401 Unauthorized');

                $exception = new AccessDeniedException('invalid password');
                echo $template_factory->render('access_denied_exception', compact('exception'));
            }
        }

        return $user_id;
    }

    public function infobox_action()
    {
        global $user;

        header('Content-type: text/html; charset=windows-1252');
        $template = $this->template_factory->open('infobox');
        $template->user = get_username($user->id);
        $template->key = $this->get_user_key($user->id);

        echo $template->render('infobox');
    }

    public function export_action($username = NULL)
    {
        global $user;

        $type = Request::option('type', 'ALL_EVENTS');
        $key = Request::option('key');

        $user_id = $this->check_user_access($username, $key);

        if (isset($user_id)) {
            if (!is_object($user) || $user->id == 'nobody') {
                $user = new Seminar_User($user_id);
            }

            $export = new CalendarExport(new CalendarWriterICalendar());
            $export->exportFromDatabase($user_id, 0, PHP_INT_MAX, 'ALL_EVENTS', NULL);

            header('Content-Type: text/calendar; charset=UTF-8');
            echo $export->getExport();
        }
    }
}
?>
